#!/usr/local/bin/perl

# make_sticky_label.pl
#
# Author: Steve Foley
# Date Started: 10 Jan 2002
# Last Modified: 14 Jan 2002
# Last Modified: 16 Jan 2002 Change default printer for Melville
# Last Modified: 17 May 2002 Added default date, repeat printing,
#			     input for SCG ID (Dan)

#
# This script generates a round, sticky, CD Stomper-compatible, CD
# label in the upper left corner of the label sheet. It is intended to
# make label generation easy to do using default values from the cruise.
#
# Default values are set in /SCG.PROC/sproc/SET.cruise.dirs,
# /scgscg/files/CRUISE.LEG
#
# Use Instructions:
# - Verify the first section of values below is correct.
# - Make sure the printer is loaded properly with a label in the upper
#     left CD spot. You can always flip the paper upside down to print the
#     other label.
# - Simply run this script. When prompted, either enter information or
#     hit return for the default value (seeded from the environment).
#     a date MUST be entered each time.
# - To print a second copy, simply do an lp on the ps file again by hand,
#   or answer (y) when prompted.
# Future Work: Make it command line based with arguments? Print two labels?

use PostScript::TextBlock;
use PostScript::Metrics;
use Env qw(CRUISE ChiSci CompTech);

# Handy Globals/Constants
$CRUISE_ENV_FILE = '/scgscg/files/CRUISE.LEG';
# $PS_OUTPUT_DIR = '/SCG.PROC/sproc/cd_rom_scripts/LABELS/';
$PS_OUTPUT_DIR = '/SCG.PROC/CRUISE.DATA/CD_LABELS/';
#$PS_OUTPUT_DIR = '/Young_Minds/TEMP.dir/';
$PS_OUTPUT_FILE = "";
$SEQUENCE_FILE = '/SCG.PROC/sproc/cd_rom_scripts/Sequence_no';
#$output_printer = "hp-1600c"; # default set here Revelle default
$output_printer = "hp2600"; # default set here Melville default
$UPPER_LEFT_CD_CENTER_X = 206;

# Environment variables as in /scgscg/files/CRUISE.LEG
$SIC_ENV_LABEL = 'ChiSci';
$SCG_ENV_LABEL = 'CompTech';
$CRUISE_ENV_LABEL = 'CRUISE';
$SHIP_ENV_LABEL = "shipname";

# Setup some style sheets
%SIO_STYLE = (font=>'Times-Italic', size=>12);
%GROUP_STYLE = (font=>'Times-Roman', size=>18);
%SHIP_STYLE = (font=>'Times-Bold', size=>18);
%SIC_STYLE = (font=>'Times-Bold', size=>12);
%SCG_STYLE = (font=>'Times-Bold', size=>12);
%SCGID_STYLE = (font=>'Times-Roman', size=>18);
%DATATYPE_STYLE = (font=>'Times-Bold', size=>12);
%SICNAME_STYLE = (font=>'Times-Roman', size=>10);
%SCGNAME_STYLE = (font=>'Times-Roman', size=>10);
%DATE_STYLE = (font=>'Times-Roman', size=>12);
%CRUISENAME_STYLE = (font=>'Times-Bold', size=>16);


# Common fixed texts
$REVELLE_TEXT = "R/V Roger Revelle";
$MELVILLE_TEXT = "R/V Melville";
$GROUP_TEXT = "Shipboard Computer Group";
#$SIC_TEXT = "SIC:";
$SIC_TEXT = "ChSci:";
$SCG_TEXT = "SCG:";
$SCGID_TEXT = "ID: ______________";

    # Ask for the SCG ID
print "SCG ID Label [___________]: ";
$response = <STDIN>;
if ($response ne "\n")  {
#    chomp($SCGID_TEXT = "ID:  $response");
    chomp($SCGID_TEXT = "$response");
} 

$SIO_TEXT = "Scripps Institution of";
$SIO2_TEXT = "Oceanography";

$EM120_TEXT = "Simrad EM120\nMultibeam Data";
$SB2K_TEXT = "SeaBeam 2000\nMultibeam Data";
$SB2K_FINAL = "SeaBeam 2000\nEntire Cruise";
$CRZDAT_TEXT = "Cruise Data";
$CTD_TEXT = "CTD Data";
$OS75_TEXT = "OS-75 Data";
$SEISMIC_TEXT = "SEISMIC FILES";
$ECHO_TEXT = "3.5kHz FILES";

# Some fixed text blocks
$SHIP_BLOCK = new PostScript::TextBlock;
$SHIP_BLOCK->addText (text=>$SHIP_TEXT, %SHIP_STYLE);
$GROUP_BLOCK = new PostScript::TextBlock;
$GROUP_BLOCK->addText (text=>$GROUP_TEXT, %GROUP_STYLE);
$SIC_BLOCK = new PostScript::TextBlock;
$SIC_BLOCK->addText (text=>$SIC_TEXT, %SIC_STYLE);
$SCG_BLOCK = new PostScript::TextBlock;
$SCG_BLOCK->addText (text=>$SCG_TEXT, %SCG_STYLE);
$SCGID_BLOCK = new PostScript::TextBlock;
$SCGID_BLOCK->addText (text=>$SCGID_TEXT, %SCGID_STYLE);
$SIO_BLOCK = new PostScript::TextBlock;
$SIO_BLOCK->addText (text=>$SIO_TEXT, %SIO_STYLE);
$SIO2_BLOCK = new PostScript::TextBlock;
$SIO2_BLOCK->addText (text=>$SIO2_TEXT, %SIO_STYLE);

# Some empty text blocks to be filled
$SICNAME_BLOCK = new PostScript::TextBlock;
$SCGNAME_BLOCK = new PostScript::TextBlock;
$DATE_BLOCK = new PostScript::TextBlock;
$CRUISENAME_BLOCK = new PostScript::TextBlock;
$TYPE_BLOCK = new PostScript::TextBlock;

# some empty globals for the user input. Needed for centering text
$SICNAME_TEXT = undef;
$SCGNAME_TEXT = undef;
$DATE_TEXT = undef;
$CRUISENAME_TEXT = undef;
$SHIP_TEXT = undef;

# MAIN BLOCK OF CODE
   # inhale the environment vars in /scgscg/files/CRUISE.LEG
   &inhale_cruise_environment;
#print "cruise: $ENV{CRUISE}, sic: $ENV{ChiSci}, scg: $ENV{CompTech}\n";

   # gather info
   &get_info_from_user;
   # generate postscript
   &generate_postscript(36, 414, $UPPER_LEFT_CD_CENTER_X);
   # output postscript
   &send_to_printer;
# END MAIN BLOCK OF CODE

#########################################################################
# prompt user for information (use cruise variables for default):
#   - em120 or cruise data?
#   - range or individual dates?
#      - start and end date   OR
#      - specific dates
#   - printer to use?

# get_info_from_user obtains the necessary values by prompting the user.
# default values are displayed to make life easier.
sub get_info_from_user
{
    my ($response, $default);

    # print the output header
    print "\nPlease enter the required information. ";
    print "(press Return for default)\n"; 

    # Ask for the ship name
    $default = &get_default($SHIP_ENV_LABEL);
    print "Ship name [$default]: ";
    $response = <STDIN>;
    if ($response eq "\n")
    {
	$SHIP_TEXT = $default; # save for centering
    } 
    else
    {
	$SHIP_TEXT = $response;
    }
    $SHIP_BLOCK->addText (text=>$SHIP_TEXT, %SHIP_STYLE);

    # Ask for the cruise name
    $default = &get_default($CRUISE_ENV_LABEL);
    print "Cruise name [$default]: ";
    $response = <STDIN>;
    if ($response eq "\n")
    {
	$CRUISENAME_TEXT = $default; # save for centering
    } 
    else
    {
	$CRUISENAME_TEXT = $response;
    }
    $CRUISENAME_BLOCK->addText (text=>$CRUISENAME_TEXT, %CRUISENAME_STYLE);

    # Ask for the scientist's name
    $default = &get_default($SIC_ENV_LABEL);
    print "Chief Scientist's name [$default]: ";
    $response = <STDIN>;
    if ($response eq "\n")
    {
	$SICNAME_TEXT = $default; # save it for centering
    } 
    else
    {
	$SICNAME_TEXT = $response; 
    }
    $SICNAME_BLOCK->addText (text=>$SICNAME_TEXT, %SICNAME_STYLE);


    # Ask for the computer tech's name
    $default = &get_default($SCG_ENV_LABEL);
    print "Computer Tech's name [$default]: ";
    $response = <STDIN>;
    if ($response eq "\n")
    {
	$SCGNAME_TEXT = $default; #save it for centering
    } 
    else
    {
	$SCGNAME_TEXT = $response; 
    }
    $SCGNAME_BLOCK->addText (text=>$SCGNAME_TEXT, %SCGNAME_STYLE);	

    # Ask for the date
    $default = &get_default($DATE_ENV_LABEL);
    print "Date (ie. \"1jan2002 to 31dec2002\") [$default]: ";
    $response = <STDIN>;
    if ($response eq "\n")
    {
	$DATE_TEXT = $default; #save it for centering
    } 
    else
    {
	$DATE_TEXT = $response;
    }
    $DATE_BLOCK->addText (text=>$DATE_TEXT, %DATE_STYLE);

    # Ask for the type of data until the user gets it right
    $default = "3";
    LOOP: while (true)
    {
	print "Select type of data:\n  1 = EM120\n  2 = Cruise Data\n  3 = SB2000\n  4 = SB2K Final\n  5 = CTD Data\n  6 = Seismic Data\n  7 = 3.5kHz\n  8 = OS-75 Data\n"  . "  [$default]: ";
	$response = <STDIN>;
	if ($response eq "1\n")
	{
	    $TYPE_BLOCK->addText (text=>$EM120_TEXT, %DATATYPE_STYLE);
	    last LOOP;
	} 
	elsif ($response eq "2\n")
	{
	    $TYPE_BLOCK->addText (text=>$CRZDAT_TEXT, %DATATYPE_STYLE);
	    last LOOP;
	}
	elsif (($response eq "\n") || ($response eq "3\n"))
	{
	    $TYPE_BLOCK->addText (text=>$SB2K_TEXT, %DATATYPE_STYLE);
	    last LOOP;
	}
	elsif ($response eq "4\n")
	{
	    $TYPE_BLOCK->addText (text=>$SB2K_FINAL, %DATATYPE_STYLE);
	    last LOOP;
	}
	elsif ($response eq "5\n")
	{
	    $TYPE_BLOCK->addText (text=>$CTD_TEXT, %DATATYPE_STYLE);
	    last LOOP;
	}
	elsif ($response eq "6\n")
	{
	    $TYPE_BLOCK->addText (text=>$SEISMIC_TEXT, %DATATYPE_STYLE);
	    last LOOP;
	}
	elsif ($response eq "7\n")
	{
	    $TYPE_BLOCK->addText (text=>$ECHO_TEXT, %DATATYPE_STYLE);
	    last LOOP;
	}
	elsif ($response eq "8\n")
	{
	    $TYPE_BLOCK->addText (text=>$OS75_TEXT, %DATATYPE_STYLE);
	    last LOOP;
	}
    }
    
}


#########################################################################
# generate the postscript
# confirm before sending to the printer

# generate_postscript executes write() methods on all of the text blocks 
# and outputs the result to a cd label file. This should fit onto a round
# sticky label.
# Params:
#   xbase The x coordinate (postscript units) for the left edge of the CD
#   ybase The y coordinate (postscript units) for the bottom edge of the CD
#   center The x coordinate (postscript units) of the center of the CD
sub generate_postscript
{
    my ($xbase, $ybase, $center) = @_;
    my ($outfile, $psCode, $psRemainder);
    
    # setup
    $outfile = $PS_OUTPUT_DIR . &get_ps_output_filename;
    open (OUT, ">$outfile") 
	or die "Could not open output file: $outfile";
    print OUT "%!PS-Adobe-2.0\n";
    print OUT "%%Creator: Steve Foley's CD label maker\n";
    print OUT "%%DocumentResources: font " . 
	"Times-Roman Times-Bold Times-Italic\n";
    print OUT "%%Pages: 1\n";
    print OUT "%%EndPrologue\n";
    print OUT "%%Page 1 1\n";
    

    # write fixed stuff
    ($psCode, $psRemainder) = 
	$GROUP_BLOCK->Write(203, 22,
			    &center_it($center, $GROUP_TEXT, %GROUP_STYLE), 
			    260 + $ybase);
    print OUT $psCode;

    ($psCode, $psRemainder) = $SIC_BLOCK->Write(36, 18, 46 + $xbase, 
						207 + $ybase);
    print OUT $psCode;

    ($psCode, $psRemainder) = $SCG_BLOCK->Write(36, 18, 46 + $xbase,
						158 + $ybase);
    print OUT $psCode;

    ($psCode, $psRemainder) = 
	$SCGID_BLOCK->Write(256, 24,
			    &center_it($center, $SCGID_TEXT, %SCGID_STYLE), 
			    100 + $ybase);
    print OUT $psCode;

    ($psCode, $psRemainder) = 
	$SIO_BLOCK->Write(146, 16,
			  &center_it($center, $SIO_TEXT, %SIO_STYLE), 
			  320 + $ybase);
    print OUT $psCode;
    
    ($psCode, $psRemainder) = 
	$SIO2_BLOCK->Write(100, 16,
			   &center_it($center, $SIO2_TEXT, %SIO_STYLE), 
			   306 + $ybase);
    print OUT $psCode;

    # write variable fields
    ($psCode, $psRemainder) = 
	$SHIP_BLOCK->Write(150, 24, 
			   &center_it($center, $SHIP_TEXT, %SHIP_STYLE), 
			   282 + $ybase);
    print OUT $psCode;


    ($psCode, $psRemainder) = 
	$SICNAME_BLOCK->Write(103, 16, 
			      &center_it(91, $SICNAME_TEXT, %SICNAME_STYLE),
			      190 + $ybase);
    print OUT $psCode;
    
    ($psCode, $psRemainder) = 
	$SCGNAME_BLOCK->Write(171, 16,
			      &center_it(91, $SCGNAME_TEXT, %SCGNAME_STYLE),
			      140 + $ybase);
    print OUT $psCode;
			      
    ($psCode, $psRemainder) = 
	$DATE_BLOCK->Write(180, 26, 
			   &center_it(210, $DATE_TEXT, %DATE_STYLE), 
			   50 + $ybase);
    print OUT $psCode;

    ($psCode, $psRemainder) = 
	$CRUISENAME_BLOCK->Write(100, 18,				 
				 &center_it($center, 
					    $CRUISENAME_TEXT, 
					    %CRUISENAME_STYLE), 
				 77 + $ybase);

    print OUT $psCode;

    ($psCode, $psRemainder) = $TYPE_BLOCK->Write(94, 40, 235 + $xbase, 
						 194 + $ybase);
    print OUT $psCode;


    # clean up
    print OUT "\nshowpage\n";
    print OUT "%%Trailer\n";
    print OUT "%%EOF\n";

    close (OUT);
    print "\nOutput stored in $outfile\n";
    system "/usr/dt/bin/sdtimage $outfile";
}


# send_to_printer gets printer information and sends the postscript file 
# to the specified printer.
sub send_to_printer
{
    my ($response, $outfile);

    $outfile = $PS_OUTPUT_DIR . &get_ps_output_filename;

    print "Printer [$output_printer]: ";
    $response = <STDIN>;
    if ($response ne "\n")
    {
	chomp($output_printer = $response);
    }  

    while (1) {   # add repeat printing 
	print "\nReady to print label to $output_printer? ([y]/n) ";
	$response = <STDIN>;
	if ( $response !~ /^N|^n/ ) {
	    # do the printing here
	    system "/bin/lp -d" . $output_printer . " " . $outfile;
	}
	else {
	    last;
	}
    }
}


#########################################################################
# Utility methods
#########################################################################

# get_default obtains the default value for the specified environment
# variable by getting it from the environment settings right now.
sub get_default
{
    my ($varname) = @_;
    return $ENV{$varname};
}

# inhale_cruise_environment reads the file with cruise environment variables
# and loads them if the required environment variables are not already set.
# This does NOT return anything, but modifies %ENV
sub inhale_cruise_environment
{
    my ($leg, $cru, $ship, $chiSci, $compTech, $item,$dateBeg, $dateEnd, 
	$year);
    # see if values are set already
    chop($year = `/usr/bin/date '+%Y'`);
    if (($ENV{$CRUISE_ENV_LABEL}   ne "")
	&& ($ENV{$SIC_ENV_LABEL}   ne "")
	&& ($ENV{$SCG_ENV_LABEL}   ne "")
	&& ($ENV{$SHIP_ENV_LABEL}  ne "")
	&& ($ENV{$BDATE_ENV_LABEL} ne "")
	&& ($ENV{$EDATE_ENV_LABEL} ne ""))
    {
	return; # already in the environment
    }
    
    # if not, open file
    open (ENVFILE, $CRUISE_ENV_FILE) or die "Could not find $CRUISE_ENV_FILE";

    # loop over lines in file, looking for the ones we care about.
    # for chief sci, try to find just the last name
    while (<ENVFILE>)
    {
	# parse for value
	if (($item) = /\w+ LEG (\d{2})/) { $leg = $item; next; }
#	next if (($leg) = /\w+ LEG (\d{2})/);
	if (($item) = /\w+ CRU (\w{4})/) { $cru = $item; next; }
	if (($item) = /\w+ SHIP (\w{2})/) { $ship = $item; next; }
	if (($item) = /\w+ ChiSci '(.+)'/) {
	    $chiSci = $item; next; 
	}
	if (($item) = /\w+ CompTech '(.+)'/) { $compTech = $item; next; }
	if (($item) = /\w+ BDATE '(.+)'/) { 
	    $dateBeg = $item; 
	    $dateBeg =~ s/-(\w+)-\d\d$/$1$year/;
	    next; 
	}
	if (($item) = /\w+ EDATE '(.+)'/) { 
	    $dateEnd = $item;   # dd-mmm-yy
	    $dateEnd =~ s/-(\w+)-\d\d$/$1$year/;
	    next; 
	}
    }

    close (ENVFILE);

    # push to environment
    $ENV{$CRUISE_ENV_LABEL} = $cru . $leg . $ship;
    $ENV{$SIC_ENV_LABEL} = $chiSci;
    $ENV{$SCG_ENV_LABEL} = $compTech;
    $ENV{$DATE_ENV_LABEL} = "$dateBeg to $dateEnd";
    if ($ship eq "RR") { $ENV{$SHIP_ENV_LABEL} = $REVELLE_TEXT; }
    elsif ($ship eq "MV") { $ENV{$SHIP_ENV_LABEL} = $MELVILLE_TEXT; }
    else { $ENV{$SHIP_ENV_LABEL} = ""; }
}

# get_ps_output_filename gets the file name where the last sequence number's
# postscript file is to be stored. It is a hybrid of a fixed label and the
# sequence number.
sub get_ps_output_filename
{
    my ($seqnum);
    open (SEQ, $SEQUENCE_FILE) 
	or die "Could not open sequence file: $SEQUENCE_FILE";
    $seqnum = <SEQ>;
    chop $seqnum;
    close(SEQ);
    return "cd_sticky_label" . $seqnum . ".ps";
}

# center_it computes the x coordinate of the left edge for the given text
# that should be centered around a specific point.
# Params: 
#   center The x coordinate of the desired center of the text
#   text The text to be measured for fit
#   style The style of the text (font and size) to use. Leading is ignored.
# Returns:
#   The left x coordinate relative the the postscript origin.
sub center_it 
{
    my ($center, $text, %style) = @_;
    my ($size);

    die "Invalid center position" if ($center <= 0);

    $size = 
      int PostScript::Metrics::stringwidth($text, $style{font}, $style{size});
    $half = int ($size/2);

    die "Text too big" if ($half > $center);

    return ($center - $half);
}

